#ifdef GL_ES
precision mediump float;
#endif

varying vec2 v_texCoord;
varying vec4 v_fragmentColor;

uniform float u_dL;
uniform float u_dH;
uniform float u_dS;


void main() {

    vec4 texColor = v_fragmentColor * texture2D(CC_Texture0, v_texCoord);
    float color[3];
    float r = texColor.r;
    float g = texColor.g;
    float b = texColor.b;
    float a = texColor.a;

    //RGB 2 HSL
    float H;
    float S;
    float L;
    float max_c = max(max(r, g), b);
    float min_c = min(min(r, g), b);

    //L
    L = 0.5 * (max_c + min_c);

    //S
    if (L < 0.5)
    {
        S = (max_c - min_c) / (max_c + min_c);
    }
    else if (L >= 0.5)
    {
        S = (max_c - min_c) / (2.0 - max_c - min_c);
    }

    //H
    if (r == max_c)
    {
        H = (g - b) / (max_c - min_c);
    }
    else if (g == max_c)
    {
        H = 2.0 + (b - r) / (max_c - min_c);
    }
    else if (b == max_c)
    {
        H = 4.0 + (r - g) / (max_c - min_c);
    }

    L = min(1.0, max(0.0, L * (1.0 + u_dL)));
    H = H * 60.0;
    H = (H * (1.0 + u_dH));
    if (H < 0.0)
    {
        H += 360.0;
    }
    else if (H > 360.0)
    {
        H -= 360.0;
    }
    S = min(1.0, max(0.0, S * (1.0 + u_dS)));


    //HSL 2 RGB
    if(S == 0.0)
    {
        color[0] = L;
        color[1] = L;
        color[2] = L;
    }
    else
    {
        float tmp1;
        float tmp2;
        float tmp[3];

        if (L < 0.5)
        {
            tmp2 = L * (1.0 + S);
        }
        else if (L >= 0.5)
        {
            tmp2 = L + S - (L * S);
        }
        tmp1 = 2.0 * L - tmp2;
        H = H / 360.0;

        tmp[0]= H + (1.0 / 3.0);
        tmp[1]= H;
        tmp[2]= H - (1.0 / 3.0);
        for(int i = 0; i < 3; i++)
        {
            if(tmp[i] < 0.0)
            {
                tmp[i] += 1.0;
            }
            else if(tmp[i] > 1.0)
            {
                tmp[i] -= 1.0;
            }

            if(6.0 * tmp[i] < 1.0)
            {
                color[i] = tmp1 + (tmp2 - tmp1) * 6.0 * tmp[i];
            }
            else if(2.0 * tmp[i] < 1.0)
            {
                color[i] = tmp2;
            }
            else if(3.0 * tmp[i] < 2.0)
            {
                color[i] = tmp1 + (tmp2 - tmp1) * ((2.0 / 3.0) - tmp[i]) * 6.0;
            }
            else
            {
                color[i] = tmp1;
            }
        }
    }

    gl_FragColor = vec4(color[0], color[1], color[2], texColor.a);
}
